<?php
/*
 * © Copyright 2007, 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage adding or editing identification details to the database.
 * 
 * @package iHRIS
 * @subpackage Manage
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2007, 2008 IntraHealth International, Inc. 
 * @since v2.0.0
 * @version v2.0.0
 */

/**
 * Page object to handle the adding or editing identification details to the database.
 * 
 * @package iHRIS
 * @subpackage Manage
 * @access public
 */
class iHRIS_PageFormMakeOffer extends iHRIS_PageFormParentPerson {
        
    /**
     * Return the form name for this page.
     * @param boolean $html Set to true if this is to be used for the html template page.
     * @return string
     * @see PersonPageForm
     */
    protected function getForm( $html=false ) {
        if ( $html && $this->getParent()->isActive() ) {
            return "make_promotion";
        }else {
            return "make_offer"; 
        }
    }

    /**
     * @var iHRIS_Position $old_position The old position this person held and is being promoted from.
     */
    private $old_position;


    /**
     * Load the HTML template files for editing.
     */
    protected function loadHTMLTemplates() {
        parent::loadHTMLTemplates();
        if ( $this->getParent()->isActive()  && $this->isPost() && !$this->post_exists('person_position_position') ) {
            $node = $this->template->getElementById('new_person_position');
            if ($node instanceof DOMElement) {
                $node->removeAttribute('noedit');
            }
        }
        if ( !$this->getParent()->isActive()  && $this->isPost() && $this->post('person_position_position') == "" ) {
            $node = $this->template->getElementById('set_person_position');
            if ($node instanceof DOMElement) {
                $node->removeAttribute('noedit');
            }
        }
    }

    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function loadObjects() {
        $pers_pos = $this->factory->createForm( "person_position", $this->id );
        if( $this->get_exists('position') ) {
            $pers_pos->position = $this->get('position');
        }
        $this->setObject( $pers_pos , I2CE_PageForm::EDIT_PRIMARY, 'new_position' );
        $salary = $this->factory->createForm( "salary" ); 
        $this->setObject( $salary, I2CE_PageForm::EDIT_CHILD );  
        parent::loadObjects(); 
        $salary->start_date = $pers_pos->start_date; 
        $this->getParent()->populateLast( array(  "person_position" => "start_date" ) ); 
        $personPosition = $this->getParent()->getLastPosition();
        if ( $this->getParent()->isActive() && $personPosition instanceof iHRIS_PersonPosition ) {

            $this->old_position = $this->factory->createForm( "position", $this->getParent()->getLastPosition()->position );                    
            $this->old_position->statusOnly();

            if ( $this->isPost() ) {
                $this->old_position->setStatus( $_POST[ $this->old_position->getField('status')->getHTMLName() ] );
                $personPosition->reason = $this->getPrimary()->reason;
            }
            $this->old_position->getField('status')->setMap( "position", "lookupStatus", "listDepartureOptions" );
        } else {
            $this->old_position = false;
        }
    }
        
    /**
     * Set the I2CE_Form object in the page template.
     * 
     * This method will pass the edit object to the page template so that it can process all the form variables.
     */
    protected function setForm() {
        parent::setForm();
        if ( $this->old_position ) {
            $this->template->setForm( $this->old_position , 'current_position' );
            $this->template->setForm( $this->getParent()->getLastPosition(), 'current_position' ); 
        }
    }
    /**
     * Populate any list drop downs and load any JavaScript files for dynamic form interaction.
     * @param boolean $hasData Some drop downs may change based on the selected value so this will set those up correctly.
     */
    protected function setDynamicForm( $hasData = false ) {
        parent::setDynamicForm( $hasData );
        if ( $this->old_position ) {
            $this->old_position->setFormLists( $this->template, $hasData , 'current_position' );
            $this->getParent()->getLastPosition()->setFormLists( $this->template, $hasData ,'current_position');
        }
    }

    /**
     * Run extra validation for the fields being edited.
     */
    protected function validate() {
        $personPosition = $this->getParent()->getLastPosition();
        parent::validate();
        if ( $this->isPost() && $this->old_position instanceof iHRIS_Position) {
        	
            if ( $this->getPrimary()->start_date->compare( $this->getParent()->getLastPosition()->start_date ) != -1 ) {
                $this->template->setFormInvalid( $this->getPrimary()->getField('start_date')->getHTMLName(), "Please enter a start date after the previous position's start date." );
            }
            if ( !$this->old_position->getField('status')->isValid() ) {
                $this->template->setFormInvalid( $this->old_position->getField('status')->getHTMLName(), "Please enter a new status." );                                
            }

            if (!$this->getPrimary()->getField('reason')->isValid()) {
                $this->template->setFormInvalid( $this->getPrimary()->getField('reason')->getHTMLName(), "Required Field" );
            }


        }
    }

    /**
     * Display the save or confirm buttons as needed.
     * 
     * If the page is a confirmation view then the save / edit button template will be displayed.  
     * Otherwise the confirm and return buttons will be shown.
     * @param boolean $save Flag to show the save button. (Defaults to false)
     * @param boolean $show_edit (defaults to true)
     * @global array
     */
    protected  function displayControls( $save = false, $show_edit = true ) {
        parent::displayControls( $save, $show_edit );
        $button_return = $this->template->getElementById( "button_return" );
        if ( $button_return ) {
            $button_return->setAttribute( "name", "position" );
            $button_return->setAttribute( "href", "manage?action=review&position=" );
        }
    }
        
    /**
     * Update the position for this to mark it as closed and then save the object.
     */ 

    public function save() {
        $position = $this->factory->createForm( "position", $this->getPrimary()->position );
        $position->statusOnly();
        $position->closePosition( $this->user, $this->getParent()->getId() );           
        if ( $this->getParent()->isActive() && $this->old_position instanceof  iHRIS_Position ) {
            $this->old_position->save( $this->user );

            $person =& $this->getParent(); 
            $personPosition = $person->getLastPosition();
            if ($personPosition instanceof iHRIS_PersonPosition) {
                $personPosition->end_date = $this->getPrimary()->start_date;
                $personPosition->reason = $this->getPrimary()->reason;
                $personPosition->save( $this->user );
                $personPosition->populateChildren('salary');
                if (is_array($personPosition->children['salary'])) { //there may be no salaries
                    foreach ($personPosition->children['salary'] as $i=> $salary) {
                        if (!$salary->end_date->isValid()) {
                            $salary->end_date = $this->getPrimary()->start_date;
                            $salary->save( $this->user );
                        }
                    }
                }
            }          
            $this->getPrimary()->reason = null;         
        }               
        parent::save();
    }
                        
}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
