<?php
/*
 * © Copyright 2007, 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Manage editing of all the position databases used for drop down menus.
 * 
 * @package iHRIS
 * @subpackage Manage
 * @access public
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2007, 2008 IntraHealth International, Inc. 
 * @since v2.0.0
 * @version v2.0.0
 */

/**
 * Page object to handle the management of all the position databases used for drop down menus.
 * 
 * This page handles all the editing of the list database tables using the relevant
 * {@link I2CE_Form} interface objects.  The base page lists all the types of data
 * that can be edited.  From there you can add a new entry or edit an existing one.
 * 
 * This object overrides the default {@link save()} and {@link action()} methods since there are many
 * database objects to be edited.
 * @package iHRIS
 * @subpackage Manage
 * @access public
 */
class iHRIS_PageFormPosition extends iHRIS_PageFormLists {

    /**
     * Create a new instance of this page.
     * 
     * This will call the parent constructor and then setup the base
     * template pages for the {@link I2CE_Template template}.  It also sets up the values
     * for the member variables.
     * @param string $title The title for this page.
     * @param string $defaultHTMLFile The default HTML file for this page.
     * @param mixed $access The role required to access this page.
     * @param array $files The list of template files to load for this page.
     */
    public function __construct( $args,$request_remainder) {
        parent::__construct($args,$request_remainder);
        if ( $this->type != "" && $this->type != "position" ) {
            $this->setAccess ( 'hr_manager' );
        }
    }
        
    /**
     * Load the HTML template files for editing and confirming the list information.
     * 
     * Since this page has special versions, this method is only called when a particular list object
     * is being added or edited.  All other pages are loaded within the {@link action} method.
     */
    protected function loadHTMLTemplates() {
        $this->template->addFile( "position_form_base.html" );
        $this->template->addFile( $this->getPrimary()->getHTMLTemplate(), "tbody" );
    }
        
    /**
     * Create and load data for the objects used for this form.
     * 
     * Create the list object and if this is a form submission load
     * the data from the form data.  It determines the type based on the
     * {@link $type} member variable.
     */
    protected function loadObjects() {
        parent::loadObjects();
        if ( $this->type == "position" && ( $this->get('submit') == "discontinue" || $this->get('submit') == "open" ) ) {
            $this->getPrimary()->statusOnly();
        }
                
    }
        
    /**
     * Display the save or confirm button templates as needed.
     * @param boolean $save Flag to show the save button.
     * @param boolean $show_edit (defaults to true)
     */
    protected function displayControls( $save = false, $show_edit = true ) {
        if ( $save ) {
        	parent::displayControls( $save, $show_edit );
        } else {
            $this->template->addFile( "button_confirm_position.html" );     
        }
    }
        
    /**
     * Save the objects to the database.
     * 
     * Save the list object being edited and return to the appropriate admin page.
     */
    protected function save() {
        parent::save();
        $this->setRedirect( "position?type=" . $this->type );
    }
        
        
    /**
     * Perform the actions of the page.
     * 
     * This handles some special actions because there are three versions of this page:
     * - The list page of all objects that can be edited.
     * - The add/update page for each list object.
     * - The edit form for each list object.
     * 
     * Only in the third case is the parent object action method called since that is the default setup
     * for editing objects used in most other {@link PageForm} objects.
     */
    protected function action() {
        if ( $this->type == "" ) {
            $this->template->addFile( "position.html" );
        } elseif ( !$this->isPost() && !$this->get_exists('id') && !$this->get_exists('add') ) {
        	
            $this->loadObjects();
            if ( $this->type == "position" ) {
                $node = $this->template->addFile( "position_select.html" );
                $this->template->addFormWorm( 'position_form' );
                //$pos_select = new iHRIS_PositionSelect();
                $factory = I2CE_FormFactory::instance();
                $pos_select = $factory->createForm( "position_select" );
                $pos_select->setFormLists( $this->template, true );
                $this->template->setForm( $pos_select, $node );
            } else {
                $this->template->addFile( "position_type_" . $this->getPrimary()->getListType() . ".html" );
                if ( $this->getPrimary()->getListType() == "list" ) {
                    $this->template->appendFileById( "position_type_header.html", "th", "lists_header" );
                    $list = I2CE_List::listOptions( $this->type );
                    foreach( $list as $record => $value ) {
                        $imported_row =& $this->template->appendFileById( "position_type_row.html", "tr", "lists_body" );
                        if ( ++$cnt % 2 == 1 ) $this->template->setNodeAttribute( "class", "even", $imported_row );
                        $this->template->setDisplayData( "add_lists_id", array("id" => $record , "type" => $this->type) );
                        $this->template->setDisplayData( "lists_row_name", $value );
                    }
                } elseif ( $this->getPrimary()->getListType() == "dual" ) {
                    $head = $this->template->appendFileById( "position_type_header.html", "th", "lists_header" );
                    $this->template->setNodeAttribute( "colspan", "2", $head );
                    $list = $this->getPrimary()->getOptionsByLink();
                    $link_form = "";
                    $add_link_rows = array();
                    foreach( $list as $link => $options ) {
                        if ( $link_form == "" ) $link_form = I2CE_List::lookupFormName( $link );
                        $link_value = I2CE_List::lookup( $link, $link_form );
                        $add_link_rows[ $link_value ] = array();
                        foreach( $options as $record => $value ) {
                            $add_link_rows[ $link_value ][$record] = $value;
                        }
                    }
                                        
                    $link_rows = array_keys( $add_link_rows );
                    sort( $link_rows );
                    foreach( $link_rows as $link_value ) {
                        foreach( $add_link_rows[$link_value] as $record => $value ) {
                            $imported_row =& $this->template->appendFileById( "position_type_dual_row.html", "tr", "lists_body" );
                            if ( ++$cnt % 2 == 1 ) $this->template->setNodeAttribute( "class", "even", $imported_row );
                            $this->template->setDisplayData( "add_lists_id", array("id" => $record , "type=" => $this->type) );
                            $this->template->setDisplayData( "lists_row_name", $value );
                            $this->template->setDisplayData( "lists_row_link", $link_value );                                           
                        }
                    }
                                        
                } else {
                    I2CE_FormField::listArrayOptions($this->template, "add_ids", I2CE_List::listOptions($this->type) );
                }
            }
            $this->setDisplayData();
            $this->setForm();
        	
        } elseif( $this->get('submit') == "discontinue" || $this->get('submit') == "open" ) {
            $this->loadObjects();
            $this->getPrimary()->setStatus( $this->get('submit') == "discontinue" ? iHRIS_Position::STATUS_DISCONTINUED : iHRIS_Position::STATUS_OPEN );
            $this->getPrimary()->save( $this->user );
            $this->setRedirect( "position?type=" . $this->type );
        } else {
            parent::action();
        }
        $this->setMenus();
    }

    /**
     * Set the attributes for the left hand navigation menu to highlight the proper location.
     */
    protected function setMenus() {
        $this->template->setAttribute( "class", "active", "menuPosition", "a[@href='position']" );
    }

}


# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
