<?php
/*
 * © Copyright 2007, 2008 IntraHealth International, Inc.
 * 
 * This File is part of iHRIS
 * 
 * iHRIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @package iHRIS
 * @subpackage Manage
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @copyright Copyright &copy; 2007, 2008 IntraHealth International, Inc. 
 * @since v1.0.0
 * @version v2.0.0
 */
/**
 * Object for dealing with positions.
 * 
 * @package iHRIS
 * @subpackage Manage
 * @access public
 */
class iHRIS_Position extends I2CE_List {
        
    /**
     * The main field name used for display a description of a record.
     */
    const MAIN_FIELD = "code";
    /**
     * The secondary field name used for displaying a description of a record in combination with the MAIN_FIELD.
     */
    const SEC_FIELD = "title";
    /**
     * Constant value for open position status.
     */
    const STATUS_OPEN = 1;
    /**
     * Constant value for closed position status.
     */
    const STATUS_CLOSED = 2;
    /**
     * Constant value for discontinued position status.
     */
    const STATUS_DISCONTINUED = 3;

    /**
     * @var array The list of position status.
     */
    static private $status = array( self::STATUS_OPEN => 'Open', self::STATUS_CLOSED => 'Closed', 
                                    self::STATUS_DISCONTINUED => 'Discontinued' );

    /**
     * Create a new instance of a position.
     * 
     * @param string $form The name of this form
     * @param integer $id
     */
    public function __construct( $form, $id = 0 ) {
        parent::__construct( $form, $id );

        $this->proposed_hiring_date = I2CE_Date::now();
        $this->proposed_end_date = I2CE_Date::blank();               
        $this->posted_date = I2CE_Date::now();
        $this->fields['status']->setValue( self::STATUS_OPEN );        
        $this->fields['type']->setValue( $form );
        
    }
    
    /**
     * Remove all fields except for the status and type for when only dealing with status updates.
     */
    public function statusOnly() {
        $this->clearFields( array( 'status', 'type' ) );
    }
    
    /**
     * Lookup the given value from the status array.
     * @param integer $id
     * @param string $form Not used for this method.
     * @return string
     */
    static public function lookupStatus( $id, $form="" ) {
        return I2CE_Form::lookupArray( $id, self::$status );
    }
    /**
     * List all the options from the status array.
     * @param string $form The  form we wish to lookup by
     * @returns array
     */
    static public function listStatusOptions($form ) {
        return self::$status;
    }
    /**
     * List all options except closed for the status arra
     * @param string $form The  form we wish to lookup byy.
     * @returns array
     */
    static public function listDepartureOptions($form ) {
        $status = self::$status;
        unset( $status[self::STATUS_CLOSED] );
        return $status;
    }

    /**
     * List all positions by status (the linked form for this object).
     * 
     * This will return an array that includes all status codes with all positions that
     * are in that status.
     * @return array
     */
    static public function getOptionsByLink() {
        $factory = I2CE_FormFactory::instance();
        return parent::_getOptionsByLink( $factory->getFormName( get_class() ), self::MAIN_FIELD, "status", self::SEC_FIELD );
    }
    
    /**
     * List all open positions
     * @param string $form The class being listed.
     * @returns array
     */
    static public function listOpen (  $form="position" ) {
        return I2CE_List::listOptions( $form, "status", 1 );
    }

    /**
     * Update the status for this position.
     * @param integer $status
     */
    public function setStatus( $status ) {
        $this->fields['status']->setValue( $status );
    }
        
    /**
     * Mark this position as closed and remove it from any applications.
     * 
     * The person being assigned the position will have all application positions removed.
     * @param I2CE_User &$user The user performing this action.
     * @param integer $person_id The person being assigned this position.
     */
    public function closePosition( &$user, $person_id ) {
        $factory = I2CE_FormFactory::instance();
        $this->setStatus( self::STATUS_CLOSED );
        $this->save( $user );
        if ( I2CE_ModuleFactory::instance()->isEnabled( "ihris-manage-Application" ) ) {
            $apps = $factory->callStatic( "person", "findApplicants", array( $this->id ) );
            if ( count( $apps ) > 0 ) {
                foreach ( $apps as $per_id => $per_name ) {
                    $app_per = $factory->createForm( "person", $per_id );
                    $app_per->populateChildren( array( "application" ) );
                    if( array_key_exists( 'application', $app_per->children ) &&
                            count( $app_per->children['application'] ) > 0 ) {
                        foreach( $app_per->children['application'] as $app ) {
                            $app->removePosition( $this->getId() );
                            $app->save( $user );
                        }
                    }
                }
                /*
                $form_id = I2CE_Form::getFormId( "application" );
                $sth = $this->db->prepare( "SELECT id,parent FROM record WHERE parent IN ( " 
                                        . implode( ",", array_keys( $apps ) ) . " ) ", null, MDB2_PREPARE_RESULT );
                if ( !I2CE::pearError( $sth, "Error preparing statement to close a position: " ) ) {
                    $app_ids = $sth->execute();
                    if ( !I2CE::pearError( $app_ids, "Error executing statement to close a position: " ) ) {
                        while( $data = $app_ids->fetchRow() ) {
                            $app = $factory->createForm( "application", $data->id );
                            $app->populate();
                            $app->removePosition( $this->id );
                            $app->save( $user );
                        }
                    }
                }
                */
            }
        }
    }
        
    /**
     * Return the list of record ids that have this position as supervisor.
     * @return array
     */
    public function getSupervised() {
        $sup_det = I2CE_FormField::getFormFieldIdAndType( "position", "supervisor" );
        if ( !$sup_det ) return array();
        $supervised = array();
        
        $db = MDB2::singleton();
        if ( !I2CE::pearError( $db, "Error getting database connection: " ) ) {
        
            $sth = $db->prepare( "SELECT record FROM last_entry WHERE form_field = ? AND " . $sup_det['type'] . "_value = ?",
                                 array('integer', 'integer' ), MDB2_PREPARE_RESULT );
            if ( !I2CE::pearError( $sth, "Error preparing statement to get supervised positions: " ) ) {
                $results = $sth->execute( array( $sup_det['id'], $this->id ) );
                if ( !I2CE::pearError( $results, "Error executing statement to get supervised positions: " ) ) {
                    while( $row = $results->fetchRow() ) {
                        $supervised[] = $row->record;
                                        
                    }
                }
            }
                        
        }
                
        return $supervised;
        
    }
        
}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
